# -*- coding: utf-8 -*-

from mininet.net import Mininet
from mininet.node import RemoteController
from mininet.node import Node
from mininet.topo import Topo
from mininet.link import TCLink
from mininet.cli import CLI 
from mininet.log import setLogLevel, info
import MccTopo

def mccNet():
	
	# Creo un 'riferimento' al controller pyretic remoto
	pyreticController = RemoteController( "controller_mcc", "127.0.0.1", 6633 )
	
	# Creo un oggetto rete con topologia MCC e mi collego al controller
	info( "--- Creo la rete e mi collego con il controller MCC Pyretic.\n" )
	net = Mininet( topo = MccTopo.MccTopo(), controller = pyreticController, link = TCLink )

	# Da codice sorgente node.py di Mininet, controllo che il controller sia agganciato
	listening = pyreticController.cmd( "echo A | telnet -e A %s %d" % ( pyreticController.ip, pyreticController.port ) )
	if 'Connected' not in listening:
		info("\n-X- Controller non trovato! Assicurarsi di averlo attivato\n\n")
	else:
		info( "--- Controller agganciato con successo.\n" )
		pyreticController.start()

	'''
	info( "--- Creo la topologia di rete.\n" )
	topologiaMcc = MccTopo.MccTopo()
	'''
	
	# Le interfacce di rete esistono già da quando i link sono stati creati nella topologia
	info( "--- Imposto gli indirizzi IP.\n" )
	for i in range(0,len(net.hosts)):

		if net.hosts[i].name == "hMeoc":
			net.hosts[i].setIP("10.0.1.33", 24, "hMeoc-eth0")	
			net.hosts[i].setMAC("00:00:00:00:02:10", "hMeoc-eth0")
			#net.hosts[i].setDefaultRoute("hMeoc-eth0")

		elif net.hosts[i].name == "h1":
			net.hosts[i].setIP("10.0.1.1", 24, "h1-eth0")
			net.hosts[i].setIP("10.0.2.1", 24, "h1-eth1")
			net.hosts[i].setMAC("00:00:00:00:01:01", "h1-eth0")
			net.hosts[i].setMAC("00:00:00:00:02:01", "h1-eth1")
			net.hosts[i].cmd("arp -i h1-eth0 -s 10.0.2.100 00:00:00:00:02:10")
			net.hosts[i].setHostRoute("10.0.2.100", "h1-eth0")

			net.hosts[i].cmd("brctl addbr br1")
			net.hosts[i].cmd("ifconfig h1-eth0 0.0.0.0 down")
			net.hosts[i].cmd("ifconfig h1-eth1 0.0.0.0 down")
			net.hosts[i].cmd("brctl addif br1 h1-eth0")
			net.hosts[i].cmd("brctl addif br1 h1-eth1")
			net.hosts[i].cmd("ifconfig h1-eth0 up")
			net.hosts[i].cmd("ifconfig h1-eth1 up")
			net.hosts[i].cmd("ifconfig br1 up")
			net.hosts[i].cmd("ifconfig br1 10.0.1.1")
			net.hosts[i].cmd("arp -i br1 -s 10.0.2.100 00:00:00:00:02:10")

		elif net.hosts[i].name == "h2":
			net.hosts[i].setIP("10.0.1.2", 24, "h2-eth0")
			net.hosts[i].setIP("10.0.2.2", 24, "h2-eth1")
			net.hosts[i].setMAC("00:00:00:00:01:02", "h2-eth0")
			net.hosts[i].setMAC("00:00:00:00:02:02", "h2-eth1")

			net.hosts[i].cmd("brctl addbr br2")
			net.hosts[i].cmd("ifconfig h2-eth0 0.0.0.0 down")
			net.hosts[i].cmd("ifconfig h2-eth1 0.0.0.0 down")
			net.hosts[i].cmd("brctl addif br2 h2-eth0")
			net.hosts[i].cmd("brctl addif br2 h2-eth1")
			net.hosts[i].cmd("ifconfig h2-eth0 up")
			net.hosts[i].cmd("ifconfig h2-eth1 up")
			net.hosts[i].cmd("ifconfig br2 up")
			net.hosts[i].cmd("ifconfig br2 10.0.1.2")

		elif net.hosts[i].name == "h3":
			net.hosts[i].setIP("10.0.1.3", 24, "h3-eth0")
			net.hosts[i].setIP("10.0.2.3", 24, "h3-eth1")
			net.hosts[i].setMAC("00:00:00:00:01:03", "h3-eth0")
			net.hosts[i].setMAC("00:00:00:00:02:03", "h3-eth1")

			net.hosts[i].cmd("brctl addbr br3")
			net.hosts[i].cmd("ifconfig h3-eth0 0.0.0.0 down")
			net.hosts[i].cmd("ifconfig h3-eth1 0.0.0.0 down")
			net.hosts[i].cmd("brctl addif br3 h3-eth0")
			net.hosts[i].cmd("brctl addif br3 h3-eth1")
			net.hosts[i].cmd("ifconfig h3-eth0 up")
			net.hosts[i].cmd("ifconfig h3-eth1 up")
			net.hosts[i].cmd("ifconfig br3 up")
			net.hosts[i].cmd("ifconfig br3 10.0.1.3")

		elif net.hosts[i].name == "h4":
			net.hosts[i].setIP("10.0.1.4", 24, "h4-eth0")
			net.hosts[i].setIP("10.0.2.4", 24, "h4-eth1")
			net.hosts[i].setMAC("00:00:00:00:01:04", "h4-eth0")
			net.hosts[i].setMAC("00:00:00:00:02:04", "h4-eth1")

			net.hosts[i].cmd("brctl addbr br4")
			net.hosts[i].cmd("ifconfig h4-eth0 0.0.0.0 down")
			net.hosts[i].cmd("ifconfig h4-eth1 0.0.0.0 down")
			net.hosts[i].cmd("brctl addif br4 h4-eth0")
			net.hosts[i].cmd("brctl addif br4 h4-eth1")
			net.hosts[i].cmd("ifconfig h4-eth0 up")
			net.hosts[i].cmd("ifconfig h4-eth1 up")
			net.hosts[i].cmd("ifconfig br4 up")
			net.hosts[i].cmd("ifconfig br4 10.0.1.4")

		elif net.hosts[i].name == "h5":
			net.hosts[i].setIP("10.0.1.5", 24, "h5-eth0")
			net.hosts[i].setIP("10.0.2.5", 24, "h5-eth1")
			net.hosts[i].setMAC("00:00:00:00:01:05", "h5-eth0")
			net.hosts[i].setMAC("00:00:00:00:02:05", "h5-eth1")

			net.hosts[i].cmd("brctl addbr br5")
			net.hosts[i].cmd("ifconfig h5-eth0 0.0.0.0 down")
			net.hosts[i].cmd("ifconfig h5-eth1 0.0.0.0 down")
			net.hosts[i].cmd("brctl addif br5 h5-eth0")
			net.hosts[i].cmd("brctl addif br5 h5-eth1")
			net.hosts[i].cmd("ifconfig h5-eth0 up")
			net.hosts[i].cmd("ifconfig h5-eth1 up")
			net.hosts[i].cmd("ifconfig br5 up")
			net.hosts[i].cmd("ifconfig br5 10.0.1.5")

		elif net.hosts[i].name == "hSink":
			net.hosts[i].setIP("10.0.2.100", 24, "hSink-eth0")
			net.hosts[i].setMAC("00:00:00:00:02:10", "hSink-eth0")
			net.hosts[i].setHostRoute("10.0.1.1", "hSink-eth0")
			net.hosts[i].setHostRoute("10.0.2.1", "hSink-eth0")
			net.hosts[i].setIP("10.0.2.102", 24, "hSink-eth1")
			net.hosts[i].setMAC("00:00:00:00:02:12", "hSink-eth1")
			net.hosts[i].setHostRoute("10.0.1.2", "hSink-eth1")
			net.hosts[i].setHostRoute("10.0.2.2", "hSink-eth1")
			net.hosts[i].setIP("10.0.2.103", 24, "hSink-eth2")
			net.hosts[i].setMAC("00:00:00:00:02:13", "hSink-eth2")
			net.hosts[i].setHostRoute("10.0.1.3", "hSink-eth2")
			net.hosts[i].setHostRoute("10.0.2.3", "hSink-eth2")
			net.hosts[i].setIP("10.0.2.104", 24, "hSink-eth3")
			net.hosts[i].setMAC("00:00:00:00:02:14", "hSink-eth3")
			net.hosts[i].setHostRoute("10.0.1.4", "hSink-eth3")
			net.hosts[i].setHostRoute("10.0.2.4", "hSink-eth3")
			net.hosts[i].setIP("10.0.2.105", 24, "hSink-eth4")
			net.hosts[i].setMAC("00:00:00:00:02:15", "hSink-eth4")
			net.hosts[i].setHostRoute("10.0.1.5", "hSink-eth4")
			net.hosts[i].setHostRoute("10.0.2.5", "hSink-eth4")
			net.hosts[i].setDefaultRoute("hSink-eth0")

			net.hosts[i].cmd("brctl addbr brSink")
			net.hosts[i].cmd("ifconfig hSink-eth0 0.0.0.0 down")
			net.hosts[i].cmd("ifconfig hSink-eth1 0.0.0.0 down")
			net.hosts[i].cmd("ifconfig hSink-eth2 0.0.0.0 down")
			net.hosts[i].cmd("ifconfig hSink-eth3 0.0.0.0 down")
			net.hosts[i].cmd("ifconfig hSink-eth4 0.0.0.0 down")
			net.hosts[i].cmd("brctl addif brSink hSink-eth0")
			net.hosts[i].cmd("brctl addif brSink hSink-eth1")
			net.hosts[i].cmd("brctl addif brSink hSink-eth2")
			net.hosts[i].cmd("brctl addif brSink hSink-eth3")
			net.hosts[i].cmd("brctl addif brSink hSink-eth4")
			net.hosts[i].cmd("ifconfig hSink-eth0 up")
			net.hosts[i].cmd("ifconfig hSink-eth1 up")
			net.hosts[i].cmd("ifconfig hSink-eth2 up")
			net.hosts[i].cmd("ifconfig hSink-eth3 up")
			net.hosts[i].cmd("ifconfig hSink-eth4 up")
			net.hosts[i].cmd("ifconfig brSink up")
			net.hosts[i].cmd("ifconfig brSink 10.0.2.100")
	
			net.hosts[i].cmd("arp -i brSink -s 10.0.1.1 00:00:00:00:01:01")


	info( "--- Attivo la rete.\n" )
	net.start()
	
	info( "--- Accedo all'interfaccia CLI.\n" )
	CLI( net )

	info ( "--- Disattivo la rete.\n" )
	net.stop()

	info ( "--- Stacco il Controller.\n" )
	pyreticController.stop()

	info ( "--- Tutto chiuso. Ciao Ciao!\n" )

if __name__ == "__main__":
	setLogLevel( "info" )
	mccNet()
