# -*- coding: utf-8 -*-

from pyretic.lib.corelib import *
from pyretic.lib.std import *
from pyretic.lib.query import *

# MAC forwarding tables. MAC -> OUTPUT PORT
class static_switch(Policy):

	# Costruttore
	def __init__(self):

		# Costruttore del padre sulla mia istanza
		super(static_switch, self).__init__()

		# fwd_table["00:00:00:00:01:33"] -> 1
		self.fwd_table = { "00:00:00:00:01:33":1 , "00:00:00:00:01:01":2 , "00:00:00:00:01:02":3 , "00:00:00:00:01:03":4 , "00:00:00:00:01:04":5 , "00:00:00:00:01:05":6 }

	# Aggiorna la policy pyretic in base alla fwd_table dell'istanza. 
	def static_MAC_forwarding_policy(self):
		
		subpolicies = []

		# Creo una policy per ogni entry nella fwd_table
		for MAC in self.fwd_table:
			subpolicies.append( match(dstmac=EthAddr(MAC)) >> fwd( self.fwd_table[MAC] ) )

		# Aggiungo anche questa per far propagare le ARP queries
		subpolicies.append( match(dstmac=EthAddr("ff:ff:ff:ff:ff:ff")) >> parallel([xfwd(1), xfwd(2), xfwd(3), xfwd(4), xfwd(5), xfwd(6)]) )

		return parallel(subpolicies)


# Policy dinamica per dare porte diverse ai pacchetti che arrivano
class mcc_wheel(DynamicPolicy):

	# Costruttore
	def __init__(self):

		# Costruttore del padre sulla mia istanza
		super(mcc_wheel, self).__init__()		

		# Contatori che uso per variare la porta in output dei pacchetti
		self.wheel = -1
		self.wheelOut = 3
		self.counterUMTS = 0
		self.counterHSDPA = 0

		# Creo una istanza di static_switch
		self.mcc_static_switch_test = static_switch()

		# Inizializzo la policy mcc
		self.policy_mcc = self.mcc_static_switch_test.static_MAC_forwarding_policy()

		self.set_initial_state()

	def set_network(self,network):
		self.set_initial_state()

	def set_initial_state(self):
		# Query su ogni pacchetto che arriva
		self.query = packets(0)
		self.query.register_callback(self.mcc_forwarding_logic)
		self.update_policy()

	# Query callback
	def mcc_forwarding_logic(self, packet):
		self.wheel += 1

		# ~Weighted Round-Robin
		if (self.wheelOut == 3):
			#1
			pass
		elif (self.wheelOut == 4):
			#2 - anche il prossimo pacchetto dovrà uscire da questa porta
			if (self.counterUMTS == 0):
				self.wheel -= 1
				self.counterUMTS += 1
			elif (self.counterUMTS == 1):
				self.counterUMTS -= 1
		elif (self.wheelOut == 5):
			#3 - anche i prossimi due pacchetti dovranno uscire da questa porta
			if (self.counterHSDPA == 0):
				self.wheel -= 1
				self.counterHSDPA += 1
			elif (self.counterHSDPA == 1):
				self.wheel -= 1
				self.counterHSDPA += 1
			elif (self.counterHSDPA == 2):
				self.counterHSDPA = 0
		elif (self.wheelOut == 6):
			#1
			pass
		

		self.wheelOut = (self.wheel % 4) + 3

		self.policy_mcc = if_( match(srcip=IPAddr("10.0.1.1")) & ~match(protocol=1) & ~match(protocol=2), fwd(self.wheelOut), self.mcc_static_switch_test.static_MAC_forwarding_policy())
	
		self.update_policy()

	# Aggiorna la policy dinamica
	def update_policy(self):
		self.policy = self.query + self.policy_mcc


# Il controller deve semplicemente ritornare la policy
def main():
	return mcc_wheel()


